#!/bin/ksh

#**************************************************************
# Author: Sebastien Denvil, Martial Mancip
# Contact: Sebastien.Denvil@ipsl.jussieu.fr Martial.Mancip@ipsl.jussieu.fr
# $Date$
# $Author$
# $Revision$
# IPSL (2006)
#  This software is governed by the CeCILL licence see libIGCM/libIGCM_CeCILL.LIC
# History:
# Modification:
#
#**************************************************************

#========================================================================
function IGCM_comp_Initialize
{
    IGCM_debug_PushStack "IGCM_comp_Initialize"

    # Debug Print :
    echo
    IGCM_debug_Print 1 "IGCM_comp_Initialize :"
    echo

    typeset comp compname comptagname CompatibilityTag auxprint card_UserChoices first_option
    for comp in ${config_ListOfComponents[*]} ; do
        # Debug Print
	IGCM_debug_Print 1 ${comp}
	# Define component 
        IGCM_card_DefineArrayFromOption ${SUBMIT_DIR}/config.card ListOfComponents ${comp}
	eval compname=\${config_ListOfComponents_${comp}[0]} > /dev/null 2>&1
	eval comptagname=\${config_ListOfComponents_${comp}[1]} > /dev/null 2>&1

	# Read libIGCM compatibility version in ${compname}.card
	card=${SUBMIT_DIR}/COMP/${compname}.card
	IGCM_card_DefineVariableFromOption ${card} Compatibility libIGCM
	eval CompatibilityTag=\${${compname}_Compatibility_libIGCM} > /dev/null 2>&1	

	if [ ! "${CompatibilityTag}" = "${libIGCM_CurrentTag}" ] ; then
	    IGCM_debug_Exit "${compname}.card is not compatible with libIGCM version ${libIGCM_CurrentTag} see libIGCM FAQ http://wiki.ipsl.jussieu.fr/wiki_ipsl/IGCMG/libIGCM/DocUtilisateur/FAQ ."
	fi

	# Manage component executable
	IGCM_card_DefineArrayFromOption ${SUBMIT_DIR}/config.card Executable ${comp}

	# Read component Write Frequency in config.card
	IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card ${comp} WriteFrequency
	# Debug Print :
	eval auxprint=\${config_${comp}_WriteFrequency}
	IGCM_debug_Print 1 "Write frequency for ${compname} : ${auxprint} "
        #2> /dev/null

	# Debug Print :
	IGCM_debug_Print 1 "Initialize following component library :"
	IGCM_debug_Print 1 ${SUBMIT_DIR}/COMP/${compname}.driver
	IGCM_debug_Print 3 "With tag : ${comptagname}"
	# Source component library
	. ${SUBMIT_DIR}/COMP/${compname}.driver

	# Debug Print
	IGCM_debug_Print 3 "Initialize ${comp} output directory."
	# Define Dirs	
	eval R_OUT_${comp}=${R_SAVE}/${comp}
	eval IGCM_sys_MkdirArchive \${R_OUT_${comp}}

	eval R_OUT_${comp}_O=\${R_OUT_${comp}}/Output
	eval IGCM_sys_MkdirArchive \${R_OUT_${comp}_O}

	eval R_OUT_${comp}_R=\${R_OUT_${comp}}/Restart
	eval IGCM_sys_MkdirArchive \${R_OUT_${comp}_R}

	eval R_OUT_${comp}_D=\${R_OUT_${comp}}/Debug
	eval IGCM_sys_MkdirArchive \${R_OUT_${comp}_D}

 	eval R_OUT_${comp}_O_I=\${R_OUT_${comp}_O}/INS
 	eval R_OUT_${comp}_O_H=\${R_OUT_${comp}_O}/HF
 	eval R_OUT_${comp}_O_D=\${R_OUT_${comp}_O}/DA
 	eval R_OUT_${comp}_O_M=\${R_OUT_${comp}_O}/MO
	eval R_OUT_${comp}_O_Y=\${R_OUT_${comp}_O}/YE


	# Read UserChoices section of component card
	IGCM_debug_Print 1 "DefineArrayFromOption  : ${compname}_UserChoices ${card}"
	IGCM_card_DefineArrayFromSection ${card} UserChoices
	eval first_option=\${${compname}_UserChoices[0]} > /dev/null 2>&1
	if [ X${first_option} != X"Error:" ] ; then
	    if [ X${card_UserChoices[0]} != X ] ; then
		unset card_UserChoices
	    fi
	    eval set +A card_UserChoices -- \${${compname}_UserChoices[*]} > /dev/null 2>&1
	    typeset option
	    for option in ${card_UserChoices[*]} ; do
		IGCM_card_DefineVariableFromOption ${card} UserChoices ${option}
		eval IGCM_debug_Print 2 "${compname}_UserChoices_values: ${option} \${card_UserChoices_${option}}"
	    done
	fi

	# Debug Print
	IGCM_debug_Print 3 "Initialize ${compname} with driver."
	# INIT component
	${comp}_Initialize
	echo
    done
    echo "-----"

    echo
    IGCM_debug_Print 1 "DefineArrayFromOption  : ListOfComponents"
    IGCM_debug_PrintVariables 3 config_ListOfComponents

    IGCM_debug_PopStack "IGCM_comp_Initialize"
}

#=======================================================================
function IGCM_comp_PrepareDeletedFiles
{
    IGCM_debug_PushStack "IGCM_comp_PrepareDeletedFiles" $@
    
    if [ X${2} != X. ] ; then
	eval FileToBeDeleted[${#FileToBeDeleted[@]}]=$( basename ${2} ) > /dev/null 2>&1
    else
	eval FileToBeDeleted[${#FileToBeDeleted[@]}]=$( basename ${1} ) > /dev/null 2>&1
    fi

    IGCM_debug_PopStack "IGCM_comp_PrepareDeletedFiles"
}

#=======================================================================
function IGCM_comp_GetInputInitialStateFiles
{
    IGCM_debug_PushStack "IGCM_comp_GetInputInitialStateFiles"

    # Debug Print :
    echo
    IGCM_debug_Print 1 "IGCM_comp_GetInputInitialStateFiles :"
    echo

    # Only the first time step need InitialStateFiles 
    # otherwise it's BoundaryConditions
    if ( ${FirstInitialize} ) ; then
      typeset comp compname comptagname card ListFilesName FileName0 NbFiles i i_ 
      typeset file_in_ file_in file_out_ file_out do_init 
      for comp in ${config_ListOfComponents[*]} ; do
	  # Initialize
	  do_init="y"
	  # Do we need to bring initial state file for this component
	  if [ "${config_Restarts_OverRule}" = "y" ] ; then
	      eval do_init="n"
	  else
	      # Read component Restarts parameters
	      IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card ${comp} Restart
	      eval do_start=\${config_${comp}_Restart} > /dev/null 2>&1
	      if [ "${do_start}" = "y" ] ; then
		    do_init="n"
	      else
		    do_init="y"
	      fi
	  fi

	  if [ "${do_init}" = "y" ] ; then
	      # Define component 
	      eval compname=\${config_ListOfComponents_${comp}[0]} > /dev/null 2>&1
	      eval comptagname=\${config_ListOfComponents_${comp}[1]} > /dev/null 2>&1

	      # Debug Print :
	      IGCM_debug_Print 3 "Initialisation files ${compname}"

	      card=${SUBMIT_DIR}/COMP/${compname}.card

	      IGCM_card_DefineArrayFromOption ${card} InitialStateFiles List
	      ListFilesName=${compname}_InitialStateFiles_List

	      eval FileName0=\${${ListFilesName}[0]} > /dev/null 2>&1
	      if [ X${FileName0} != X${NULL_STR} ] ; then
		  eval NbFiles=\${#${ListFilesName}[@]} > /dev/null 2>&1

		  (( i=0 ))
		  until [ $i -ge $NbFiles ]; do
		      eval file_in_=\${${ListFilesName}[$i]} > /dev/null 2>&1
		      eval file_in=${file_in_}
		      (( i_ = i+1 ))
		      eval file_out_=\${${ListFilesName}[$i_]} > /dev/null 2>&1
		      eval file_out=${file_out_}
		      
		      IGCM_sys_Get ${file_in} ${file_out}

		      IGCM_comp_PrepareDeletedFiles ${file_in} ${file_out}

		      (( i=i+2 ))
		  done
	      fi
	  fi
      done
    fi
    IGCM_debug_PopStack "IGCM_comp_GetInputInitialStateFiles"
}

#=======================================================================
function IGCM_comp_GetInputBoundaryFiles
{
    IGCM_debug_PushStack "IGCM_comp_GetInputBoundaryFiles"

    # Debug Print :
    echo
    IGCM_debug_Print 1 "IGCM_comp_GetInputBoundaryFiles :"
    echo

    typeset comp compname comptagname card ListFilesName FileName0 NbFiles i i_ 
    typeset file_in_ file_in file_out_ file_out 

    if [ ${Period} = 1 ]; then
	ListFixBoundary=" "
    fi

    for comp in ${config_ListOfComponents[*]} ; do

	echo "-----"
	# Define component
	eval compname=\${config_ListOfComponents_${comp}[0]} > /dev/null 2>&1
	eval comptagname=\${config_ListOfComponents_${comp}[1]} > /dev/null 2>&1

        # Debug Print :
	IGCM_debug_Print 3 "Boundary files ${compname}"

	card=${SUBMIT_DIR}/COMP/${compname}.card

	IGCM_card_DefineArrayFromOption ${card} BoundaryFiles List
	ListFilesName=${compname}_BoundaryFiles_List
 	eval FileName0=\${${ListFilesName}[0]} > /dev/null 2>&1

	if [ X${FileName0} != X${NULL_STR} ] ; then
	    eval NbFiles=\${#${ListFilesName}[@]} > /dev/null 2>&1

	    (( i=0 ))
	    until [ $i -ge $NbFiles ]; do
		eval file_in_=\${${ListFilesName}[$i]} > /dev/null 2>&1
		eval file_in=${file_in_}
		(( i_ = i+1 ))
		eval file_out_=\${${ListFilesName}[$i_]} > /dev/null 2>&1
		eval file_out=${file_out_}

		IGCM_sys_Get ${file_in} ${file_out}

		IGCM_comp_PrepareDeletedFiles ${file_in} ${file_out}

		(( i=i+2 ))
	    done
	fi

	# Get non deleted files
	if [ ${Period} = 1 ]; then

	    IGCM_card_DefineArrayFromOption ${card} BoundaryFiles ListNonDel
	    ListFilesName=${compname}_BoundaryFiles_ListNonDel
	    eval FileName0=\${${ListFilesName}[0]} > /dev/null 2>&1
	    
	    if [ X${FileName0} != X${NULL_STR} ] ; then
		eval NbFiles=\${#${ListFilesName}[@]} > /dev/null 2>&1
		
		(( i=0 ))
		until [ $i -ge $NbFiles ]; do
		    eval file_in_=\${${ListFilesName}[$i]} > /dev/null 2>&1
		    eval file_in=${file_in_}
		    (( i_ = i+1 ))
		    eval file_out_=\${${ListFilesName}[$i_]} > /dev/null 2>&1
		    eval file_out=${file_out_}

		    IGCM_sys_Get ${file_in} ${file_out}

		    if [ X${file_out} != X. ] ; then
			ListFixBoundary=${ListFixBoundary}" "${file_out}
		    else
			ListFixBoundary=${ListFixBoundary}" "$( basename ${file_in} )
		    fi

		    (( i=i+2 ))
		done
	    fi
	fi
    done
    
    IGCM_debug_PopStack "IGCM_comp_GetInputBoundaryFiles"
}

#=======================================================================
function IGCM_comp_DelFixeBoundaryFiles
{
    IGCM_debug_PushStack "IGCM_comp_DelFixeBoundaryFiles"

    # Debug Print :
    echo
    IGCM_debug_Print 1 "IGCM_comp_DelFixeBoundaryFiles :"
    echo

    ls -l ${ListFixBoundary}
    rm -f ${ListFixBoundary}

    IGCM_debug_PopStack "IGCM_comp_DelFixeBoundaryFiles"
}

#=======================================================================
function IGCM_comp_GetInputParametersFiles
{
    IGCM_debug_PushStack "IGCM_comp_GetInputParametersFiles"

    # Debug Print :
    echo
    IGCM_debug_Print 1 "IGCM_comp_GetInputParametersFiles :"
    echo

    typeset comp compname comptagname card ListFilesName FileName0 NbFiles i i_ file_in file_out
    for comp in ${config_ListOfComponents[*]} ; do
	# Define component 
	eval compname=\${config_ListOfComponents_${comp}[0]} > /dev/null 2>&1
	eval comptagname=\${config_ListOfComponents_${comp}[1]} > /dev/null 2>&1

	# Debug Print :
	IGCM_debug_Print 3 "Parameters ${compname}"

	card=${SUBMIT_DIR}/COMP/${compname}.card	

	IGCM_card_DefineArrayFromOption ${card} ParametersFiles List
	ListFilesName=${compname}_ParametersFiles_List
 	eval FileName0=\${${ListFilesName}[0]} > /dev/null 2>&1

	if [ X${FileName0} != X${NULL_STR} ] ; then
	    eval NbFiles=\${#${ListFilesName}[@]} > /dev/null 2>&1

	    (( i=0 ))
	    until [ $i -ge $NbFiles ]; do
		eval file_in_=\${${ListFilesName}[$i]} > /dev/null 2>&1
		eval file_in=${file_in_}
		(( i_ = i+1 ))
		eval file_out_=\${${ListFilesName}[$i_]} > /dev/null 2>&1
		eval file_out=${file_out_} 

		IGCM_sys_Cp ${file_in} ${file_out} 

		IGCM_comp_PrepareDeletedFiles ${file_in} ${file_out}

		(( i=i+2 ))
	    done
	fi
    done

    IGCM_debug_PopStack "IGCM_comp_GetInputParametersFiles"
}

#=======================================================================
function IGCM_comp_GetInputRestartFiles
{
    IGCM_debug_PushStack "IGCM_comp_GetInputRestartFiles"

    # Debug Print :
    echo
    IGCM_debug_Print 1 "IGCM_comp_GetInputRestartFiles :"
    echo

    typeset Date_tmp Date_r Path_r do_start CompOldName
    typeset comp compname comptagname card ListFilesName FileName0 NbFiles i i_ 
    typeset file_in file_out file_in_ file_out_ file_in_Name

    for comp in ${config_ListOfComponents[*]} ; do
	# Define component 
	eval compname=\${config_ListOfComponents_${comp}[0]} > /dev/null 2>&1
	eval comptagname=\${config_ListOfComponents_${comp}[1]} > /dev/null 2>&1

	# Debug Print :
	IGCM_debug_Print 3 "restart ${compname}"

	card=${SUBMIT_DIR}/COMP/${compname}.card	

	if ( ${FirstInitialize} ) ; then

	    if [ "${config_Restarts_OverRule}" = "y" ] ; then
		eval config_${comp}_Restart="y"
		eval config_${comp}_RestartDate=${config_Restarts_RestartDate}
		eval config_${comp}_RestartJobName=${config_Restarts_RestartJobName}
		eval config_${comp}_RestartPath=${config_Restarts_RestartPath}
		eval do_start=\${config_${comp}_Restart} > /dev/null 2>&1
		eval CompOldName=${comp}
	    else
	        # Read component Restarts parameters
		IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card ${comp} Restart
		eval do_start=\${config_${comp}_Restart} > /dev/null 2>&1

		if [ "${do_start}" = "y" ] ; then
		    IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card ${comp} RestartDate
		    IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card ${comp} RestartJobName
		    IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card ${comp} RestartPath
		else
		    eval config_${comp}_RestartDate=-1
		    eval config_${comp}_RestartJobName=${NULL_STR}
		    eval config_${comp}_RestartPath=${NULL_STR}
		fi
		IGCM_card_DefineVariableFromOption ${SUBMIT_DIR}/config.card ${comp} OldName
		eval CompOldName=\${config_${comp}_OldName}
		if [ X${CompOldName} = X ] ; then
		    eval CompOldName=${comp}
		fi
	    fi

	    if [ "${do_start}" = "y" ] ; then

		IGCM_card_DefineArrayFromOption ${card} RestartFiles List
		ListFilesName=${compname}_RestartFiles_List
		eval FileName0=\${${ListFilesName}[0]} > /dev/null 2>&1
		
		if ( [ X${FileName0} != X${NULL_STR} ] && [ X${FileName0} != XNONE ] ) ; then
		    eval NbFiles=\${#${ListFilesName}[@]} > /dev/null 2>&1

		    (( i=1 ))
		    until [ $i -gt $NbFiles ]; do
			eval file_in_=\${${ListFilesName}[$i]} > /dev/null 2>&1
			eval file_in=${file_in_}

			(( i_ = i+1 ))
			eval file_out_=\${${ListFilesName}[$i_]} > /dev/null 2>&1
			eval file_out=${file_out_}
			
			eval Date_tmp=\${config_${comp}_RestartDate}
			Date_r=$( IGCM_date_ConvertFormatToGregorian ${Date_tmp} )
			eval Path_r=\${config_${comp}_RestartPath}/\${config_${comp}_RestartJobName}/${CompOldName}/Restart
			eval file_in_Name=\${config_${comp}_RestartJobName}_${Date_r}_${file_in}

			eval IGCM_sys_Get ${Path_r}/${file_in_Name} ${file_out} 

			IGCM_comp_PrepareDeletedFiles ${file_in} ${file_out}
		
			(( i=i+3 ))
		    done
		else
		    if [ X${FileName0} != XNONE ] ; then
			IGCM_debug_Exit "IGCM_comp_GetInputRestartFiles : No file in list for ${compname}."
		    else
			IGCM_debug_Print 1 "IGCM_comp_GetInputRestartFiles : NONE specified in Restart List ${compname}."
		    fi
		fi
	    fi
	else
            # if not FirstInitialize

	    # Restore Restarts files
	    #-----------------------
	    IGCM_card_DefineArrayFromOption ${card} RestartFiles List
	    ListFilesName=${compname}_RestartFiles_List
	    eval FileName0=\${${ListFilesName}[0]} > /dev/null 2>&1
	    
	    if ( [ X${FileName0} != X${NULL_STR} ] && [ X${FileName0} != XNONE ] ) ; then
		eval NbFiles=\${#${ListFilesName}[@]} > /dev/null 2>&1

		(( i=1 ))
		until [ $i -gt $NbFiles ]; do
		    eval file_in_=\${${ListFilesName}[$i]} > /dev/null 2>&1
		    eval file_in=${file_in_}
		    (( i_ = i+1 ))
		    eval file_out_=\${${ListFilesName}[$i_]} > /dev/null 2>&1
		    eval file_out=${file_out_}
		    
		    file_in_Name=${run_Configuration_OldPrefix}_${file_in}
		    eval IGCM_sys_Get \${R_OUT_${comp}_R}/${file_in_Name} ${file_out}

		    IGCM_comp_PrepareDeletedFiles ${file_in} ${file_out}

		    (( i=i+3 ))
		done
	    else
		if [ X${FileName0} != XNONE ] ; then
		    IGCM_debug_Exit "IGCM_comp_GetInputRestartFiles : No file in list for ${compname}."
		else
		    IGCM_debug_Print 1 "IGCM_comp_GetInputRestartFiles : NONE specified in Restart List ${compname}."
		fi
	    fi
	fi
    done
    IGCM_sys_Chmod -R u+rw .

    IGCM_debug_PopStack "IGCM_comp_GetInputRestartFiles"
}

#=======================================================================
function IGCM_comp_Update
{
    IGCM_debug_PushStack "IGCM_comp_Update"

    # Debug Print :
    echo
    IGCM_debug_Print 1 "IGCM_comp_Update :"
    echo

    typeset ExeNameIn ExeNameOut 
    typeset comp compname comptagname
    for comp in ${config_ListOfComponents[*]} ; do
	# Define component 
	eval compname=\${config_ListOfComponents_${comp}[0]} > /dev/null 2>&1
	eval comptagname=\${config_ListOfComponents_${comp}[1]} > /dev/null 2>&1

	# Copy executable for this component
	eval ExeNameIn=\${config_Executable_${comp}[0]}
	eval ExeNameOut=\${config_Executable_${comp}[1]}
	if [ -f ${R_EXE}/${ExeNameIn} ]  ; then
	    eval IGCM_sys_Cp ${R_EXE}/${ExeNameIn} ${ExeNameOut}
	    if [ -f ${RUN_DIR}/${ExeNameOut} ] ; then
                eval IGCM_sys_Chmod +rx ${RUN_DIR}/${ExeNameOut}
            fi
	fi

	# Debug Print
	IGCM_debug_Print 3 "Update ${compname} Parameter Files."
	# UPDATE component
        ${comp}_Update

    done

    IGCM_debug_PopStack "IGCM_comp_Update"
}

#=======================================================================
function IGCM_comp_Finalize
{
    IGCM_debug_PushStack "IGCM_comp_Finalize"

    # Debug Print :
    echo
    IGCM_debug_Print 1 "IGCM_comp_Finalize :"
    echo

    typeset ListTextName TextName0
    typeset comp compname comptagname card ListFilesName FileName0 NbFiles 
    typeset i i_ file_in file_in_ file_out file_out_ generic_file_name nb_rebuild_file
    for comp in ${config_ListOfComponents[*]} ; do
	# Define component 
	eval compname=\${config_ListOfComponents_${comp}[0]} > /dev/null 2>&1
	eval comptagname=\${config_ListOfComponents_${comp}[1]} > /dev/null 2>&1

	# Debug Print
	IGCM_debug_Print 1 "Finalize ${compname} component."
	# FINALIZE component
        ${comp}_Finalize

	card=${SUBMIT_DIR}/COMP/${compname}.card	

	# Save Restarts files
	#--------------------
	IGCM_card_DefineArrayFromOption ${card} RestartFiles List
	ListFilesName=${compname}_RestartFiles_List
	eval FileName0=\${${ListFilesName}[0]} > /dev/null 2>&1
	
	if ( [ X${FileName0} != X${NULL_STR} ] && [ X${FileName0} != XNONE ] ) ; then
	    eval NbFiles=\${#${ListFilesName}[@]} > /dev/null 2>&1
	    
	    (( i=0 ))
	    until [ $i -ge $NbFiles ]; do
		eval file_in_=\${${ListFilesName}[$i]} > /dev/null 2>&1
		eval file_in=${file_in_}

		(( i_ = i+1 ))
		eval file_out_=\${${ListFilesName}[$i_]} > /dev/null 2>&1
		eval file_out=${file_out_}
		
		eval IGCM_sys_Put_Rest ${file_in} \${R_OUT_${comp}_R}/${config_UserChoices_JobName}_${PeriodDateEnd}_${file_out}

		eval FileToBeDeleted[${#FileToBeDeleted[@]}]="${file_in}" > /dev/null 2>&1

		(( i=i+3 ))
	    done
	else
	    if [ X${FileName0} != XNONE ] ; then
		IGCM_debug_Exit "IGCM_comp_Finalize : No file in restart list for ${compname}."
	    else
		IGCM_debug_Print 1 "IGCM_comp_Finalize : NONE specified in Restart List ${compname}."
	    fi
	fi

	# Save Output files
	#------------------
	IGCM_card_DefineArrayFromOption ${card} OutputFiles List
	ListFilesName=${compname}_OutputFiles_List
 	eval FileName0=\${${ListFilesName}[0]} > /dev/null 2>&1

	if [ X${FileName0} != X${NULL_STR} ] ; then
	    eval NbFiles=\${#${ListFilesName}[@]} > /dev/null 2>&1

	    (( i=0 ))
	    until [ $i -ge $NbFiles ]; do
		eval file_in_=\${${ListFilesName}[$i]} > /dev/null 2>&1
		eval file_in=${file_in_}
		(( i_ = i+1 ))
		eval file_out_=\${${ListFilesName}[$i_]} > /dev/null 2>&1
		eval file_out=${file_out_}

		generic_file_name=$( basename ${file_in} .nc )
		nb_rebuild_file=$( ls | grep "${generic_file_name}[_0-9]*.nc" | wc -l )
		if [ ${nb_rebuild_file} -gt 1 ] ; then
		    IGCM_debug_Print 2 "Rebuilding  ${file_in}"
		    IGCM_sys_rebuild ${file_in} ${generic_file_name}_*.nc
		    # DEBUG Rebuild :
		    #eval IGCM_sys_Cp ${generic_file_name}_*.nc \${R_OUT_${comp}_O_M}/
		elif [ -f ${generic_file_name}_0000.nc ] ; then
		    IGCM_sys_Mv ${generic_file_name}_0000.nc ${file_in}
		fi

		IGCM_sys_Put_Out ${file_in} ${file_out}
		if [ $? -eq 0 ] ; then
		    eval FileToBeDeleted[${#FileToBeDeleted[@]}]="${file_in}" > /dev/null 2>&1
		    if [ ${nb_rebuild_file} -gt 1 ] ; then
			eval FileToBeDeleted[${#FileToBeDeleted[@]}]=$( ls | grep "${generic_file_name}_[0-9]*.nc" ) > /dev/null 2>&1
		    fi
		fi
		(( i=i+3 ))
	    done
	fi

	# Save Output Text files of models
	#---------------------------------
	
	IGCM_card_DefineArrayFromOption ${card} OutputText List
	ListTextName=${compname}_OutputText_List
	
 	eval TextName0=\${${ListTextName}[0]} > /dev/null 2>&1
	if [ X${TextName0} != X${NULL_STR} ] ; then
	    eval NbFiles=\${#${ListTextName}[@]} > /dev/null 2>&1

	    (( i=0 ))
	    until [ $i -eq $NbFiles ]; do
		eval file_in=\${${ListTextName}[$i]} > /dev/null 2>&1
		(( i=i+1 ))

		nb_text_file=$( ls ${file_in}* 2>/dev/null | wc -l )
		if [ ${nb_text_file} -gt 1 ] ; then
		    list_file=$( ls ${file_in}* )
		    for file in ${list_file}
		      do
		      eval IGCM_sys_Put_Out ${file} \${R_OUT_${comp}_D}/${PREFIX}_${file}
		      eval FileToBeDeleted[${#FileToBeDeleted[@]}]="${file}" > /dev/null 2>&1
		    done
		else
		    if ( [ -f ${file_in}_0000 ] || [ -f ${file_in}0 ] ) ; then
			eval IGCM_sys_Mv ${file_in}* ${file_in}
		    fi
		    eval IGCM_sys_Put_Out ${file_in} \${R_OUT_${comp}_D}/${PREFIX}_${file_in}
		    eval FileToBeDeleted[${#FileToBeDeleted[@]}]="${file_in}" > /dev/null 2>&1
		fi
	    done
	fi
    done

    IGCM_debug_PopStack "IGCM_comp_Finalize"
}
